###*
# Copyright (c) 2013 JeongHoon Byun aka "Outsider", <http://blog.outsider.ne.kr/>
# Licensed under the MIT license.
# <http://outsider.mit-license.org/>
###

### global angular ###

angular.module('summernote', []).controller('SummernoteController', [
  '$scope'
  '$attrs'
  '$timeout'
  ($scope, $attrs, $timeout) ->
    'use strict'
    currentElement = undefined
    summernoteConfig = $scope.summernoteConfig or {}
    if angular.isDefined($attrs.height)
      summernoteConfig.height = $attrs.height
    if angular.isDefined($attrs.focus)
      summernoteConfig.focus = true
    if angular.isDefined($attrs.airmode)
      summernoteConfig.airMode = true
    if angular.isDefined($attrs.lang)
      if !angular.isDefined($.summernote.lang[$attrs.lang])
        throw new Error('"' + $attrs.lang + '" lang file must be exist.')
      summernoteConfig.lang = $attrs.lang
    summernoteConfig.oninit = $scope.init

    summernoteConfig.onenter = (evt) ->
      $scope.enter evt: evt
      return

    summernoteConfig.onfocus = (evt) ->
      $scope.focus evt: evt
      return

    summernoteConfig.onblur = (evt) ->
      $scope.blur evt: evt
      return

    summernoteConfig.onpaste = (evt) ->
      $scope.paste evt: evt
      return

    summernoteConfig.onkeyup = (evt) ->
      $scope.keyup evt: evt
      return

    summernoteConfig.onkeydown = (evt) ->
      $scope.keydown evt: evt
      return

    if angular.isDefined($attrs.onImageUpload)

      summernoteConfig.onImageUpload = (files, editor) ->
        $scope.imageUpload
          files: files
          editor: editor
          editable: $scope.editable
        return

    @activate = (scope, element, ngModel) ->

      updateNgModel = ->
        newValue = element.code()
        if ngModel and ngModel.$viewValue != newValue
          $timeout (->
            ngModel.$setViewValue newValue
            return
          ), 0
        return

      summernoteConfig.onChange = (contents) ->
        updateNgModel()
        $scope.change
          contents: contents
          editable: $scope.editable
        return
      summernoteConfig.codemirror =
        theme: 'monokai'
        mode: 'xml'

      element.summernote summernoteConfig
      editor$ = element.next('.note-editor')
      unwatchNgModel = undefined
      editor$.find('.note-toolbar').click ->
        updateNgModel()
        # sync ngModel in codeview mode
        if editor$.hasClass('codeview')
          editor$.on 'keyup', updateNgModel
          if ngModel
            unwatchNgModel = scope.$watch((->
              ngModel.$modelValue
            ), (newValue) ->
              editor$.find('.note-codable').val newValue
              return
            )
        else
          editor$.off 'keyup', updateNgModel
          if angular.isFunction(unwatchNgModel)
            unwatchNgModel()
        return
      if ngModel

        ngModel.$render = ->
          element.code ngModel.$viewValue or ''
          return

      # set editable to avoid error:isecdom since Angular v1.3
      if angular.isDefined($attrs.editable)
        $scope.editable = editor$.find('.note-editable')
      currentElement = element
      # use jquery Event binding instead $on('$destroy') to preserve options data of DOM
      element.on '$destroy', ->
        element.destroy()
        $scope.summernoteDestroyed = true
        return
      return

    $scope.$on '$destroy', ->
      # when destroying scope directly
      if !$scope.summernoteDestroyed
        currentElement.destroy()
      return
    return
]).directive 'summernote', [ ->
  'use strict'
  {
    restrict: 'EA'
    transclude: true
    replace: true
    require: [
      'summernote'
      '^?ngModel'
    ]
    controller: 'SummernoteController'
    scope:
      summernoteConfig: '=config'
      editable: '='
      init: '&onInit'
      enter: '&onEnter'
      focus: '&onFocus'
      blur: '&onBlur'
      paste: '&onPaste'
      keyup: '&onKeyup'
      keydown: '&onKeydown'
      change: '&onChange'
      imageUpload: '&onImageUpload'
    template: '<div class="summernote"></div>'
    link: (scope, element, attrs, ctrls) ->
      summernoteController = ctrls[0]
      ngModel = ctrls[1]
      summernoteController.activate scope, element, ngModel
      return

  }
 ]
