require 'find'
require 'fileutils'

class SedgeCms::PagesController < SedgeCms::ApplicationController
  before_filter :files_exist?, :page_exist?

  def show
    # render html: Rails.cache.fetch(params[:path]) { render_mustache.html_safe }
    render html: render_mustache.html_safe
  end

  def edit
    @editing = true
    render html: render_mustache.html_safe
  end

  def title
    return @title
  end

  def editing
    return @editing
  end

  def edit_path
    return "/cms/pages/#{params[:path]}"
  end

  def show_path
    return "/#{params[:path].to_s.gsub('edit', '')}"
  end

private
  # Before Filters
  def files_exist?
    @locale = 'en'
    @mustache_file_paths = []
    @summernotes = {}
    Find.find(Rails.root.join('app', 'sedge')) do |path|
      @mustache_file_paths << path if path =~ /.*\.mustache$/
    end
    @mustache_file_paths.each do |path|
      f = File.read(path)
      doc = Nokogiri::HTML(PageAPI.render(f))
      summernotes = doc.css('summernote')
      summernotes.each do |sn|

        sedge_path = Rails.root.join('app', 'sedge')
        locale_path = Rails.root.join('app', 'sedge', 'locale')

        # make locale path method
        locale_base_path = File.dirname(path.gsub(sedge_path.to_s, ''))
        full_locale_path = "#{locale_path}/#{I18n.locale}#{locale_base_path}"
        full_locale_file_path = "#{full_locale_path}/#{File.basename(path, '.*')}/#{sn['file']}.mustache"

        @summernotes[full_locale_file_path] = sn
      end
    end
    @summernotes.each do |file_path,sn|
      if !File.exist?(file_path)
        puts "creating file #{file_path}".red
        create_file(file_path)
      end
    end
  end

  def page_exist?
    if !File.exist?("#{Rails.root.join('app', 'sedge', 'pages')}/#{params[:path]}.mustache")
      render 'errors/404'
    end
  end


  # Helpers
  def create_file(path)
    dir = File.dirname(path)
    unless File.directory?(dir)
      FileUtils.mkdir_p(dir)
    end
    File.new(path, 'w')
    # git commit
  end

  def add_editbar(doc)
    # Should check if the template exist in the rails app and then the rails engine to allow
    # for template overriding / white-labeling
    snippet_path = "#{Sedge::Engine.root.join('app', 'sedge', 'snippets')}/editbar.mustache"
    snippet_file = File.read(snippet_path)
    editbar_html = Nokogiri::HTML.fragment(PageAPI.render(snippet_file))
    doc.at('body').children.first.add_previous_sibling(editbar_html)
  end

  def add_navbar(doc)
    # Should check if the template exist in the rails app and then the rails engine to allow
    # for template overriding / white-labeling
    snippet_path = "#{Sedge::Engine.root.join('app', 'sedge', 'snippets')}/editbar.mustache"
    snippet_file = File.read(snippet_path)
    editbar_html = Nokogiri::HTML.fragment(PageAPI.render(snippet_file))
    doc.inner_html = editbar_html
    # doc.at('body').children.first.add_previous_sibling(editbar_html)
  end

  def add_javascript_source(url, doc)
    Nokogiri::HTML::Builder.with(doc.at('body')) do |d|
      d.script('src' => url)
    end
  end

  def add_stylesheet_source(url, doc)
    Nokogiri::HTML::Builder.with(doc.at('head')) do |d|
      d.link('href' => url, 'rel' => 'stylesheet' )
    end
  end

  def get_snippets(mustache_file)
    regexp = /{{{snippets\/(.*)}}}/
    return mustache_file.scan(regexp).flatten
  end

  def load_summernotes(html_fragment, path)
    sedge_path = Rails.root.join('app', 'sedge')
    locale_path = Rails.root.join('app', 'sedge', 'locale')

    locale_base_path = File.dirname(path.gsub(sedge_path.to_s, ''))
    full_locale_path = "#{locale_path}/#{I18n.locale}#{locale_base_path}"

    html_fragment.css('summernote').each do |sn|
      full_locale_file_path = "#{full_locale_path}/#{File.basename(path, '.*')}/#{sn['file']}.mustache"

      # summernote_path = "#{File.dirname(path)}/#{File.basename(path, '.*')}/#{sn['file']}.mustache"
      if File.exist?(full_locale_file_path)
        summernote_file = File.read(full_locale_file_path)
        sn.inner_html = PageAPI.render(summernote_file.to_s, @snippets)
      end
    end
  end

  def load_snippets(mustache_file)
    @snippets = {}
    snippets = get_snippets(mustache_file)
    snippets.each do |snippet_name|
      path = "#{Rails.root.join('app', 'sedge', 'snippets')}/#{snippet_name}.mustache"
      if File.exist?(path)
        snippet_file = File.read(path)
        snippet_fragment = Nokogiri::HTML.fragment(PageAPI.render(snippet_file))

        if @editing
          set_summernotes(snippet_fragment, path)
        end

        load_summernotes(snippet_fragment, path)
        @snippets["snippets/#{snippet_name}"] = snippet_fragment
      end
    end
  end

  def set_summernotes(fragment, path)
    sedge_path = Rails.root.join('app', 'sedge')
    locale_path = Rails.root.join('app', 'sedge', 'locale')
    fragment.css('summernote').each do |sn|
      # locale_path = "#{locale_path}/#{I18n.locale}#{path.gsub(sedge_path.to_s, '')}"
      # puts locale_path.inspect.green
      # puts locale_path.to_s.gsub(File.dirname(path).to_s, '').red

      locale_base_path = File.dirname(path.gsub(sedge_path.to_s, ''))
      full_locale_path = "#{locale_path}/#{I18n.locale}#{locale_base_path}"
      full_locale_file_path = "#{full_locale_path}/#{File.basename(path, '.*')}/#{sn['file']}.mustache"

      # summernote_path = "#{locale_path}/#{I18n.locale}/#{File.basename(path, '.*')}/#{sn['file']}.mustache"
      puts full_locale_file_path.inspect.red
      sn['ng-model'] = "summernotes.#{full_locale_file_path.unpack('h*')[0]}"
      sn['path'] = "#{full_locale_file_path}"

    end
  end

  def load_page(page_path)
    page_mustache_file = File.read(page_path)
    load_snippets(page_mustache_file)
    page_html_fragment = Nokogiri::HTML.fragment(page_mustache_file)

    if @editing
      set_summernotes(page_html_fragment, page_path)
    end

    page_config = page_html_fragment.search('.//sedge-config').remove
    page_options = template_options(page_config)

    load_summernotes(page_html_fragment, page_path)

    page_html = PageAPI.render(page_html_fragment.to_s)
    if page_options[:layout]
      layout_path = "#{Rails.root.join('app', 'sedge', 'layouts')}/#{page_options[:layout]}.mustache"
      layout_mustache_file = File.read(layout_path)
      load_snippets(layout_mustache_file)
      layout_html_fragment = Nokogiri::HTML.fragment(layout_mustache_file)

      if @editing
        set_summernotes(layout_html_fragment, layout_path)
      end

      layout_config = layout_html_fragment.search('.//sedge-config').remove
      layout_options = template_options(layout_config)
      load_summernotes(layout_html_fragment, layout_path)
      @snippets[:body] = page_html
      html = PageAPI.render(layout_mustache_file.to_s, @snippets)
      return {:layout => {:html => html, :options => layout_options, :page => {:html => page_html, :options => page_options}}}
    end
    return {:html => page_html, :options => page_options}
  end


  def render_mustache
    page_controller = self
    PageAPI.send(:define_method, 'controller')  { page_controller }
    page_path = "#{Rails.root.join('app', 'sedge', 'pages')}/#{params[:path]}.mustache"
    page = load_page(page_path)

    if page[:layout]
      html = page[:layout][:html]
    else
      html = page[:html]
    end

    doc = Nokogiri::HTML(html)

    if @editing
      add_stylesheet_source('/assets/summernote/dist/summernote.css', doc)
      add_stylesheet_source('/assets/sedge_cms/editor.css', doc)
      add_stylesheet_source('/assets/angular-ui-tree/dist/angular-ui-tree.min.css', doc)

      doc.at('body')['ng-app'] = 'Editor'
      doc.at('body')['ng-controller'] = 'PageCtrl'
      add_javascript_source("http://cdnjs.cloudflare.com/ajax/libs/highlight.js/7.3/highlight.min.js", doc)
      add_javascript_source("/assets/sedge_cms/editor.js", doc)

      container_node = doc.create_element "div class='container-fluid'"
      row_node = doc.create_element "div class='row'"
      l_col_node = doc.create_element "div class='col-md-3'"
      c_col_node = doc.create_element "div class='col-md-6'"
      r_col_node = doc.create_element "div class='col-md-3'"

      snippet_path = "#{Sedge::Engine.root.join('app', 'sedge', 'snippets')}/sidebar.mustache"
      snippet_file = File.read(snippet_path)
      sidebar_html = Nokogiri::HTML.fragment(PageAPI.render(snippet_file))

      l_col_node.inner_html = sidebar_html

      add_widget(r_col_node, 'page_options')

      row_node << l_col_node
      row_node << c_col_node
      row_node << r_col_node

      container_node << row_node

      doc.css('div.container').each do |div|
        row_node = doc.create_element "div class='row'"
        col_node = doc.create_element "div class='col-xs-12'"
        col_node.inner_html = div.inner_html
        row_node << col_node
        div.replace row_node
        c_col_node << row_node
      end

      doc.at('body').children.first.add_previous_sibling(container_node)

    end
    # if logged in as an admin
    add_editbar(doc)

    return doc.to_s
  end

end

class PageAPI < Mustache
end
