module SedgeCms
  require 'colored'
  require 'mustache'
  require 'haml'
  require 'ostruct'

  class ApplicationController < ActionController::Base
    before_filter :build_collections

    def dashboard
    end

    def raise_not_found!
      render 'errors/404'
    end

    def collections
      return @collections
    end

    def templates
      render "templates/#{params[:path]}", :layout => nil
    end

  private
    # private api for loading a snippet/widget into a parent html node
    def add_widget(parent, section)
      snippet_path = "#{SedgeCms::Engine.root.join('app', 'sedge', 'snippets')}/#{section}.mustache"
      snippet_file = File.read(snippet_path)
      html_fragment = Nokogiri::HTML.fragment(PageAPI.render(snippet_file))
      parent << html_fragment
    end

    # private api for reading template options
    # params config: valid yaml syntax
    # returns a symbolized hash of option key values
    def template_options(config)
      options = HashWithIndifferentAccess.new(YAML.load(config.inner_html.to_s))
      return options
    end

    def build_collections
      current_uri = request.env['PATH_INFO']
      @collections = {}
      pages_path = Rails.root.join('app', 'sedge', 'pages')
      page_template_paths = Dir[ File.join(pages_path, '**', '*') ].reject { |p| File.directory? p }
      page_template_paths.each do |template_path|
        css_classes = ""
        page_template_file = File.read(template_path)
        page_template = Nokogiri::HTML.fragment(page_template_file)
        page_config = page_template.search('.//sedge-config').remove
        page_options = template_options(page_config)
        if page_options[:belongs_to]
          page_options[:belongs_to].each do |collection|
            link_path = template_path.to_s.gsub(pages_path.to_s, '').gsub('.mustache', '')
            page_options.merge!({:link_path => link_path})
            if current_uri == link_path
              page_options.merge!({:class => "active"})
            end
            if !page_options[:sort_index]
              link[:sort_index] = '9999'
            end
            if @collections[collection.to_sym]
              if !@collections[collection.to_sym].include?(page_options)
                @collections[collection.to_sym].push(page_options)
                @collections[collection.to_sym].sort_by! { |hsh| hsh[:sort_index] }
              end
            else
              @collections[collection.to_sym] = [page_options]
            end
          end
        end
      end
    end

    def directory_hash(path, name=nil)
      data = {:title => (name || path)}
      data[:is_dir] = File.directory?(path)
      data[:items] = items = []
      sedge_path = Rails.root.join('app', 'sedge')
      Dir.foreach(path) do |entry|
        next if (entry == '..' || entry == '.')
        full_path = File.join(path, entry)
        item = nil
        if File.directory?(full_path)
          item = directory_hash(full_path, entry)
        else
          sedge_path = Rails.root.join('app', 'sedge')
          edit_path = "#{File.dirname(full_path)}/#{File.basename(entry, '.*')}"
          edit_path = edit_path.gsub(sedge_path.to_s, '')
          item = {:is_dir => false, :title => File.basename(entry, '.*'), :edit_path => "/cms#{edit_path}"}
        end
        item[:id] = full_path.unpack('h*')[0]
        items << item
      end
      return data
    end

  end
end
